@@ -2,7 +2,6 @@
 # encoding: UTF-8
 
 import base64
-import re
 import hashlib
 import logging
 import math
@@ -59,7 +58,7 @@ class API(object):
     Please http capture request from (mobile) qqmusic mobile web page
     """
 
-    def __init__(self, timeout=1):
+    def __init__(self, timeout=2):
         # TODO: 暂时无脑统一一个 timeout
         # 正确的应该是允许不同接口有不同的超时时间
         self._timeout = timeout
@@ -125,10 +124,14 @@ class API(object):
         # Other supported types: songlist, user, mv, qc, gedantip, zhida.
         if type_ == 0:
             key_ = 'song'
-        elif type_ == 8:
-            key_ = 'album'
-        elif type_ == 9:
+        elif type_ == 1:
             key_ = 'singer'
+        elif type_ == 2:
+            key_ = 'album'
+        elif type_ == 3:
+            key_ = 'songlist'  # playlist
+        elif type_ == 4:
+            key_ = 'mv'  # video
         else:
             raise QQIOError('invalid search type_:%d', type_)
         payload = {
@@ -178,6 +181,25 @@ class API(object):
             return None
         return data_song
 
+    def batch_song_details(self, song_ids):
+        """
+        song_ids should be a list of int
+        """
+        payload = {
+            'comm': self.get_wkv17_common_params(),
+            'req_0': {
+                'module': 'music.trackInfo.UniformRuleCtrl',
+                'method': 'CgiGetTrackInfo',
+                'param': {
+                    'ids': song_ids,
+                    'types': [200]*len(song_ids),
+                    'source': 'AiNoFree',
+                }
+            }
+        }
+        js = self.rpc(payload)
+        return js['req_0']['data']['tracks']
+
     def song_similar(self, song_id):
         payload = {
             "simsongs": {
@@ -229,6 +251,7 @@ class API(object):
                     'singerid': artist_id,
                     'begin': (page - 1) * page_size,
                     'num': page_size,
+                    'order': 1,  # 热门/新，不带这个字段就是按歌曲新旧排序
                     # 有 newsong 字段时，服务端会返回含有 file 字段的字典
                     'newsong': 1
                 }},
@@ -311,13 +334,19 @@ class API(object):
                     'HostUin': mid
                 }},
             'comm': {
-                'g_tk': self.get_token_from_cookies(),
+                "cv": 4747474,
+                "ct": 24,
+                'g_tk': int(self.get_token_from_cookies()),
+                'g_tk_new_20200303': int(self.get_token_from_cookies()),
                 'uin': uid,
                 'format': 'json',
+                'notice': 0,
+                'platform': 'yqq.json',
+                'needNewCode': 1,
             }
         }
         js = self.rpc(payload)
-        return js['req_0']['data']['List']
+        return js['req_1']['data']['List']
 
     def user_favorite_albums(self, uid, start=0, end=100):
         url = api_base_url + '/fav/fcgi-bin/fcg_get_profile_order_asset.fcg'
@@ -351,30 +380,12 @@ class API(object):
         }
 
         resp = requests.get(url, params=params, headers=self._headers,
-                            timeout=self._timeout)
+                            cookies=self._cookies, timeout=self._timeout)
         js = resp.json()
         if js['code'] != 0:
             raise CodeShouldBe0(js)
         return js['data']['cdlist']
 
-    def get_recommend_songs_pid(self):
-        data = {
-            'req_0': {
-                'module': 'recommend.RecommendFeedServer',
-                'method': 'get_recommend_feed',
-                'param': {
-                    'direction': 0,
-                    'page': 1,
-                    'v_cache': [],
-                    'v_uniq': [],
-                    's_num': 0
-                }
-            },
-        }
-        js = self.rpc(data)
-        disstid = js['req_0']['data']['v_shelf'][0]['v_niche'][0]['v_card'][1]['id']
-        return disstid
-
     def recommend_playlists(self):
         data = {
             'recomPlaylist': {
@@ -390,24 +401,24 @@ class API(object):
         playlist = js['recomPlaylist']
         return playlist['data']['v_hot']
 
-    def get_recommend_playlists_ids(self, index=0):
+    def get_recommend_feed(self, page=1):
+        # APIs are found in https://y.qq.com/wk_v17/#/recommend
         data = {
             'req_0': {
                 'module': 'recommend.RecommendFeedServer',
                 'method': 'get_recommend_feed',
                 'param': {
                     'direction': 0,
-                    'page': 1,
+                    'page': page,
                     'v_cache': [],
                     'v_uniq': [],
                     's_num': 0
                 }
             },
+            'comm': self.get_wkv17_common_params(),
         }
         js = self.rpc(data)
-        ids = [card['id']
-               for card in js['req_0']['data']['v_shelf'][index]['v_niche'][0]['v_card']]
-        return ids
+        return js['req_0']['data']
 
     def get_lyric_by_songmid(self, songmid):
         url = api_base_url + '/lyric/fcgi-bin/fcg_query_lyric_new.fcg'
@@ -434,11 +445,27 @@ class API(object):
         }
         url = 'https://u.y.qq.com/cgi-bin/musicu.fcg'
         resp = requests.get(url, params=params, headers=self._headers,
-                            timeout=self._timeout)
+                            cookies=self._cookies, timeout=self._timeout)
         js = resp.json()
         CodeShouldBe0.check(js)
         return js
 
+    def get_wkv17_common_params(self):
+        return {
+            # ct field is important, without this field,
+            # the req_0 result is completely different.
+            "ct": 20,
+            "cv": 1770,
+            'g_tk': 5381,
+            'uin': self._uin,
+            'format': 'json',
+            'inCharset': 'utf-8',
+            'outCharset': 'utf-8',
+            'platform': 'wk_v17',
+            'uid': '',
+            'guid': '',
+        }
+
     def get_common_params(self):
         return {
             'loginUin': self._uin,
@@ -489,13 +516,34 @@ class API(object):
             },
         }
         js = self.rpc(payload)
-        res_json = []
-        for track in js['songlist']['data']['tracks']:
-            track['songid'] = track.pop('id')
-            track['songmid'] = track.pop('mid')
-            track['songname'] = track.pop('name')
-            res_json.append(track)
-        return res_json
+        return js['songlist']['data']['tracks']
+
+    def get_diss_info(self, dissid, offset=0, limit=50):
+        """获取歌单的详情
+
+        这是一种特殊的歌单，它的内容是动态的，由官方生成。比如“百万收藏”。
+
+        :param dissid: int, 举个例子 211111 是百万收藏。
+        :return: 参考 fixtures/get_diss_info.json
+        """
+        payload = {
+            'req': {
+                "module": "music.srfDissInfo.aiDissInfo",
+                "method":"uniform_get_Dissinfo",
+                "param": {
+                    "disstid":dissid,
+                    "userinfo":1,  # 不懂啥意思
+                    "tag":1,  # 不懂啥意思
+                    "orderlist":1,
+                    "song_begin": offset,
+                    "song_num": limit,
+                    "onlysonglist":0,
+                    "enc_host_uin":""  # 注：即使登录了，这个也是空
+                }
+            }
+        }
+        js = self.rpc(payload)
+        return js['req']['data']
 
     def get_song_url(self, song_mid):
         uin = self._uin
